#ifndef QUAD_H
#define QUAD_H

#include "Vector.h"

class Quad {
public:
	int IsInside(Vector s);
	int PlaneIntersection(Vector p1,Vector p2);
	int PlaneIntersection2Way(Vector p1,Vector p2);
	Vector PlaneIntersectionPoint(Vector p1,Vector p2);
	Vector BounceVector(Vector v);
	Vector BounceVector(Vector v,float res); //res=1 equivalent to bounce, res=0 equivalent to slide
	Vector SlideVector(Vector v);
	Vector v1;
	Vector v2;
	Vector v3;
	Vector v4;
	Vector n;
};

#endif

